// naumann@stce.rwth-aachen.de

#include <limits>
#include <cmath>

#include "f.hpp"

template<typename T, typename PT>
void first_derivative(T &x, const PT &p, const PT &w, T &dx) {
  T h=(x==0) ? sqrt(sqrt(std::numeric_limits<T>::epsilon()))
	     : sqrt(sqrt(std::numeric_limits<T>::epsilon()))*fabs(x);
  T xph=x+h; f(xph,p,w); 
  T xmh=x-h; f(xmh,p,w); 
  dx=(xph-xmh)/(2*h); 
  f(x,p,w);
}

template<typename T, typename PT>
void second_derivative(T &x, const PT &p, const PT &w, T &dx, T &ddx) {
  T h=(x==0) ? sqrt(sqrt(std::numeric_limits<T>::epsilon()))
	     : sqrt(sqrt(std::numeric_limits<T>::epsilon()))*fabs(x);
  T xph=x+h, dxph; first_derivative(xph,p,w,dxph); 
  T xmh=x-h, dxmh; first_derivative(xmh,p,w,dxmh); 
  ddx=(dxph-dxmh)/(2*h); 
  first_derivative(x,p,w,dx);
}

#include "h_main.hpp"
